/*
 * Copyright (c) 2007 by Brian Marick.
 * Inspired by the Java version of the Hierarchical Data format, from the
 * University of Illinois NCSA and The HDF Group (http://www.hdfgroup.org/).
 * This code is intended for use only for training in test-driven design.
 * See LICENSE.txt
 */

package exampler.hdf.internal.converters;

import exampler.hdf.internal.NumberDescription;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class IntArray_3Byte_Converter extends IntArray_4Byte_Converter {
    private final int toByteMask0, toByteMask1, toByteMask2;
    private final int toByteShift0, toByteShift1, toByteShift2;

    private final int fromByte0, fromByte1, fromByte2;

    // TODO: Not currently an error if the number overflows.

    public IntArray_3Byte_Converter(NumberDescription description) {
        super(description);
        if (getByteOrder().equals(ByteOrder.BIG_ENDIAN)) {
            toByteMask0 = 0x00FF0000; toByteShift0 = 16;
            toByteMask1 = 0x0000FF00; toByteShift1 = 8;
            toByteMask2 = 0x000000FF; toByteShift2 = 0;

            fromByte0 = 0;
            fromByte1 = 1;
            fromByte2 = 2;
        } else {
            toByteMask0 = 0x000000FF; toByteShift0 = 0;
            toByteMask1 = 0x0000FF00; toByteShift1 = 8;
            toByteMask2 = 0x00FF0000; toByteShift2 = 16;

            fromByte0 = 2;
            fromByte1 = 1;
            fromByte2 = 0;
        }

    }

    public Object convertToJavaFormat(ByteBuffer rawBytes, long lengthOfResult) {
        // Note: there's no guarantee that the byte buffer has an accessible
        // backing store. (That is,  rawBytes.array() may not work.) 
        int[] destination = new int[(int)lengthOfResult];
        int sourceIndex = 0;
        for (int destIndex = 0; destIndex < destination.length; destIndex++) {
            destination[destIndex] = threeByteToJava(rawBytes, sourceIndex);
            sourceIndex += 3;
        }
        return destination;
    }

    public void overwriteAsBytes(Object javaish, ByteBuffer destination) {
        int[] ints = (int[])javaish;
        for (int source : ints) {
            overwriteThreeBytes(destination, source);
        }

    }

    private void overwriteThreeBytes(ByteBuffer destination, int source) {
        destination.put((byte) ((source & toByteMask0) >> toByteShift0) );
        destination.put((byte) ((source & toByteMask1) >> toByteShift1) );
        destination.put((byte) ((source & toByteMask2) >> toByteShift2) );
    }


    private int threeByteToJava(ByteBuffer source, int i) {
        return (0x00FF0000 & (source.get(i+fromByte0)<<16)) |
               (0x0000FF00 & (source.get(i+fromByte1)<<8)) |
               (0x000000FF & source.get(i+fromByte2));
    }
}
