/*
 * Copyright (c) 2007 by Brian Marick.
 * Inspired by the Java version of the Hierarchical Data format, from the
 * University of Illinois NCSA and The HDF Group (http://www.hdfgroup.org/).
 * This code is intended for use only for training in test-driven design.
 * See LICENSE.txt
 */

package exampler.hdf.internal;

import exampler.hdf.CompoundDataset;
import exampler.hdf.testUtilities.TestUtils;

import java.nio.ByteOrder;

public class NumberDescriptionTest extends TestUtils {

    public void test_equality() {
        NumberDescription one = new NumberDescription(CompoundDataset.INTEGER_TYPE, 3, ByteOrder.BIG_ENDIAN);
        NumberDescription same = new NumberDescription(CompoundDataset.INTEGER_TYPE, 3, ByteOrder.BIG_ENDIAN);
        NumberDescription diffType = new NumberDescription("bogus", 3, ByteOrder.BIG_ENDIAN);
        NumberDescription diffSize = new NumberDescription(CompoundDataset.INTEGER_TYPE, 0xBADFACE, ByteOrder.BIG_ENDIAN);
        NumberDescription diffOrder = new NumberDescription(CompoundDataset.INTEGER_TYPE, 3, ByteOrder.LITTLE_ENDIAN);


        assertEquals(one, same);
        assertNotEquals(one, diffType);
        assertNotEquals(one, diffSize);
        assertNotEquals(one, diffOrder);
    }

    public void test_creation_of_copy_with_irrelevant_byte_order() {
        NumberDescription big = Like.integer(3, ByteOrder.BIG_ENDIAN);
        NumberDescription little = Like.integer(3, ByteOrder.LITTLE_ENDIAN);
             assertNotEquals(big, little);

        assertEquals(big.withoutByteOrder(), little.withoutByteOrder());

        // A description WITH a specified order does not equal one without.
        assertNotEquals(big, big.withoutByteOrder());
        assertNotEquals(little, big.withoutByteOrder());
    }
}
